# coding: utf-8
import cherrypy
import logging
import re
from cherrypy import HTTPError
from ..json import error_page_json
from ujson import loads as json_loads, dumps as json_dumps

json_out = cherrypy.tools.json_out()
logger = logging.getLogger(__name__)

__all__ = [
    'Crudi',
]


class CrudiMeta(type):

    item_id_key = '__crudi_item_id__'

    def __new__(cls, name, parents, dct):
        if '_cp_config' not in dct:
            dct['_cp_config'] = {}
        dct['_cp_config'].update({
            'error_page.default': error_page_json,
        })

        def _cp_dispatch(self, vpath):
            if hasattr(cherrypy.request, cls.item_id_key):
                raise HTTPError(404, 'Not found')
            setattr(cherrypy.request, cls.item_id_key, vpath.pop(0))
            return self
        dct['_cp_dispatch'] = _cp_dispatch

        exposed = {}
        for key in dct.keys():
            m = re.match('^(index|item)_(get|post|put|delete)(_(.*?))?$', key)
            if m is None:
                continue
            obj_method_name = m.group(4)
            if obj_method_name is None:
                obj_method_name = 'index'
            request_method = m.group(2).lower()
            unit = m.group(1)
            if obj_method_name not in exposed:
                exposed[obj_method_name] = {}
            if request_method not in exposed[obj_method_name]:
                exposed[obj_method_name][request_method] = {}
            obj_method = dct.pop(key)
            obj_method = obj_method
            exposed[obj_method_name][request_method][unit] = obj_method
        logger.info("exposed for %s is %s", name, exposed)

        for obj_method_name in exposed.keys():
            if obj_method_name in dct:
                raise TypeError("Name conflict class %s has %s name" % (name, obj_method_name))

            exposed_config = exposed[obj_method_name]

            def closure(exposed_config):
                # python closure works in such way
                def obj_method(self, *args, **kwargs):
                    logger.info("exposed: %s", exposed_config)
                    request_method = cherrypy.request.method.lower()
                    if request_method in ('post', 'put'):
                        try:
                            cherrypy.request.json = json_loads(cherrypy.request.body.decode('utf-8'))
                        except ValueError:
                            raise HTTPError(400, 'Invalid JSON document')

                    method_config = exposed_config.get(request_method)
                    if method_config is None:
                        raise HTTPError(405, 'Method not allowed')

                    final_method = None
                    unit = 'index'
                    item_key = getattr(cherrypy.request, cls.item_id_key, None)
                    if item_key is not None:
                        unit = 'item'
                    logger.info('UNIT %s METHOD CONFIG %s', unit, method_config)
                    final_method = method_config.get(unit)
                    if final_method is None:
                        raise HTTPError(404, 'Not found')
                    if item_key is not None:
                        item = self.get_item_by_id(item_key)
                        if item is None:
                            raise HTTPError(404, 'Not found')
                        args = [item] + list(args)
                    result = final_method(self, *args, **kwargs)

                    if isinstance(result, dict) or isinstance(result, list):
                        cherrypy.response.headers['Content-Type'] = 'application/json'
                        return json_dumps(result, ensure_ascii=False)
                    return result
                return obj_method

            obj_method = closure(exposed_config)
            obj_method.exposed = True

            dct[obj_method_name] = obj_method
        return super(CrudiMeta, cls).__new__(cls, name, parents, dct)


class Crudi(object):

    __metaclass__ = CrudiMeta

    def get_item_by_id(self, item_id):
        raise HTTPError(501, "Not implemented")
