# -*- coding: utf-8 -*-
import logging
from cherrypy.process.plugins import SimplePlugin
from sqlalchemy import create_engine
from sqlalchemy.orm import scoped_session, sessionmaker

__all__ = ['SqlAlchemyDbEnginePlugin']

logger = logging.getLogger(__name__)


class SqlAlchemyDbEnginePlugin(SimplePlugin):
    def __init__(self, bus, connection_string=None, orm_base=None):
        """
        The plugin is registered to the CherryPy engine and therefore
        is part of the bus (the engine *is* a bus) registery.

        We use this plugin to create the SA engine. At the same time,
        when the plugin starts we create the tables into the database
        using the mapped class of the global metadata.
        """
        SimplePlugin.__init__(self, bus)
        self.orm_base = orm_base
        self.connection_string = connection_string
        self.sa_engine = None
        self.session = scoped_session(sessionmaker(autoflush=True, autocommit=False))

    def start(self):
        self.bus.log('Starting up DB access')
        self.sa_engine = create_engine(self.connection_string, echo=False)
        if self.orm_base is not None:
            logger.info('Creating tables %s', self.sa_engine)
            self.orm_base.metadata.create_all(self.sa_engine)
        self.bus.subscribe("bind-session", self.bind)
        self.bus.subscribe("commit-session", self.commit)

    def stop(self):
        self.bus.log('Stopping down DB access')
        self.bus.unsubscribe("bind-session", self.bind)
        self.bus.unsubscribe("commit-session", self.commit)
        if self.sa_engine:
            self.sa_engine.dispose()
            self.sa_engine = None

    def bind(self):
        """
        Whenever this plugin receives the 'bind-session' command, it applies
        this method and to bind the current session to the engine.

        It then returns the session to the caller.
        """
        self.session.configure(bind=self.sa_engine)
        return self.session

    def commit(self):
        """
        Commits the current transaction or rollbacks if an error occurs.

        In all cases, the current session is unbound and therefore
        not usable any longer.
        """
        try:
            self.session.commit()
        except:
            self.session.rollback()
            raise
        finally:
            self.session.remove()
